// ARDUINO CODE FOR SMART SATA DRIVER TRANSMITTER (S.CHABOT)

#include <SPI.h>
#include <RF24.h>
#include <ezButton.h>

RF24 radio(9, 10);  // CE, CSN pins
const byte address[6] = "00006";

//define pin button values
ezButton Up_button(3, INPUT);
ezButton Right_button(4, INPUT);
ezButton Down_button(5, INPUT);
ezButton Left_button(6, INPUT);
ezButton E_button(7, INPUT);
ezButton F_button(8, INPUT);
ezButton Joy_button(2, INPUT);
ezButton Extra_button(19);
ezButton E_stop_button(18);

//Create data package to be sent
struct Data_Package {
  bool Up_button;
  bool Right_button;
  bool Down_button;
  bool Left_button;
  bool E_button;
  bool F_button;
  bool Joy_button;
  bool Extra_button;
  bool rotate_CW;
  bool rotate_CCW;
  bool bend_for;
  bool bend_back;
  bool E_stop_button;
  bool Handshake;
};

Data_Package data;  // Create a variable with the above structure


//define variables
int A = 1;
int X = 1;
int Y = 1;
int pot_X_cal;
int pot_Y_cal;
int max_X_value;
int max_Y_value;
int min_X_value;
int min_Y_value;
int pot_X_value;
int pot_Y_value;
int speed_X;
int speed_Y;
int default_value_X;
int default_value_Y;

int Up_press = 0;
int Left_press = 0;
int Down_press = 0;
int Right_press = 0;
int E_press = 0;
int F_press = 0;
int Joy_press = 0;
int Extra_press = 0;
int E_stop_press = 0;

int prev_Up_button_state;      // Variable to store the previous state of Up_button
int prev_Left_button_state;    // Variable to store the previous state of Left_button
int prev_Down_button_state;    // Variable to store the previous state of Down_button
int prev_Right_button_state;   // Variable to store the previous state of Right_button
int prev_E_button_state;       // Variable to store the previous state of E_button
int prev_F_button_state;       // Variable to store the previous state of F_button
int prev_Joy_button_state;     // Variable to store the previous state of Joystick button / Joy_button
int prev_Extra_button_state;   // Variable to store the previous state of the Extra button --> Couple button
int prev_E_stop_button_state;  // Variable to store the previous state of Emergency stop button
int prev_rotate_CW_state;
int prev_rotate_CCW_state;
int prev_bend_for_state;
int prev_bend_back_state;

unsigned long lastHandshakeTime;
unsigned long handshakeInterval = 400;

void setup() {
  Serial.begin(9600);
  radio.begin();
  radio.openWritingPipe(address);  // Set the pipe address to communicate with the receiver
  radio.setPALevel(RF24_PA_MIN);
  radio.stopListening();

  // Ground pin for extra button
  pinMode(17, OUTPUT);
  digitalWrite(17, LOW);
}

void loop() {
  unsigned long currentTime = millis();


  //CALIBRATE POTMETERS IN JOYSTICK
  while (A == 1) {
    if (analogRead(A0) < 500 && analogRead(A0) > 470 && X == 1) {
      default_value_X = analogRead(A0);
      X = 2;
    }
    if (analogRead(A1) < 520 && analogRead(A1) > 500 && Y == 1) {
      default_value_Y = analogRead(A1);
      Y = 2;
    }
    if (X == 2 && Y == 2) {
      A = 2;
    }
  }

  //activate buttons
  Up_button.loop();
  Right_button.loop();
  Down_button.loop();
  Left_button.loop();
  E_button.loop();
  F_button.loop();
  Joy_button.loop();
  Extra_button.loop();
  E_stop_button.loop();


  // Read button presses
  // PLEASE NOTE: Since we use a Joystick shield, a button press is registered as a release
  // and a release is registered as a press, due to the resistors built into the shield!!

  // Emergency_Stop_button E_stop_button
  if (E_stop_button.isPressed() && E_stop_press == 0) {
    data.E_stop_button = 1;  //Button is pressed
    E_stop_press = 1;
  } else if (E_stop_button.isReleased() && E_stop_press == 1) {
    data.E_stop_button = 0;  //Button is released
    E_stop_press = 0;
  }

  // Up_button
  if (Up_button.isReleased() && Up_press == 0) {
    data.Up_button = 1;  //Button is pressed
    Up_press = 1;
  } else if (Up_button.isPressed() && Up_press == 1) {
    data.Up_button = 0;  //Button is released
    Up_press = 0;
  }

  // Left_button
  if (Left_button.isReleased() && Left_press == 0) {
    data.Left_button = 1;  //Button is pressed
    Left_press = 1;
  } else if (Left_button.isPressed() && Left_press == 1) {
    data.Left_button = 0;  //Button is released
    Left_press = 0;
  }

  // Down_button
  if (Down_button.isReleased() && Down_press == 0) {
    data.Down_button = 1;  //Button is pressed
    Down_press = 1;
  } else if (Down_button.isPressed() && Down_press == 1) {
    data.Down_button = 0;  //Button is released
    Down_press = 0;
  }

  // Right_button
  if (Right_button.isReleased() && Right_press == 0) {
    data.Right_button = 1;  //Button is pressed
    Right_press = 1;
  } else if (Right_button.isPressed() && Right_press == 1) {
    data.Right_button = 0;  //Button is released
    Right_press = 0;
  }

  // E_button
  if (E_button.isReleased() && E_press == 0) {
    data.E_button = 1;  //Button is pressed
    E_press = 1;
  } else if (E_button.isPressed() && E_press == 1) {
    data.E_button = 0;  //Button is released
    E_press = 0;
  }

  // F_button
  if (F_button.isReleased() && F_press == 0) {
    data.F_button = 1;  //Button is pressed
    F_press = 1;
  } else if (F_button.isPressed() && F_press == 1) {
    data.F_button = 0;  //Button is released
    F_press = 0;
  }

  // Joy_button
  if (Joy_button.isPressed() && Joy_press == 0) {
    data.Joy_button = 1;  //Button is pressed
    Joy_press = 1;
  } else if (Joy_button.isReleased() && Joy_press == 1) {
    data.Joy_button = 0;  //Button is released
    Joy_press = 0;
  }

  // Extra_button
  if (Extra_button.isPressed() && Extra_press == 0) {
    data.Extra_button = 1;  //Button is pressed
    Extra_press = 1;
  } else if (Extra_button.isReleased() && Extra_press == 1) {
    data.Extra_button = 0;  //Button is released
    Extra_press = 0;
  }

  //READ POTMETERS VALUE (center position value pot_X/pot_Y is approx. 520-530)
  pot_X_value = analogRead(A0);
  pot_Y_value = analogRead(A1);


  //MAP X SPEED VALUE ACCORDING TO DEFAULT/CALIBRATED VALUE
  if (pot_X_value < default_value_X && A == 2) {
    speed_X = map(pot_X_value, 0, default_value_X - 1, 100, 0);  //100 to 0 to compensate mirrored values - map(value, fromLow, fromHigh, toLow, toHigh)
  } else if (pot_X_value >= default_value_X && A == 2) {
    speed_X = map(pot_X_value, default_value_X, 1023, 0, -100);  //0- to-100 to compensate mirrored values
  }

  //MAP Y SPEED VALUE ACCORDING TO DEFAULT/CALIBRATED VALUE
  if (pot_Y_value < default_value_Y && A == 2) {
    speed_Y = map(pot_Y_value, 0, default_value_Y - 1, 100, 0);  //100 to 0 to compensate mirrored values
  } else if (pot_Y_value >= default_value_Y && A == 2) {
    speed_Y = map(pot_Y_value, default_value_Y, 1023, 0, -100);  //0- to-100 to compensate mirrored values
  }

  // No speed for X and Y
  if (speed_X >= -10 && speed_X <= 10) {
    data.rotate_CW = 0;
    data.rotate_CCW = 0;
    // Direction Counterclockwise
  }
  if (speed_Y >= -10 && speed_Y <= 10) {
    data.bend_for = 0;
    data.bend_back = 0;
    // Direction Counterclockwise
  }

  //Categorize 3 speed levels for ROTATING in both directions (X-values)
  //But only Drive X if it is bigger than Y
  if (abs(speed_X) > abs(speed_Y)) {
    data.bend_for = 0;
    data.bend_back = 0;
    if (speed_X < -10 && speed_X >= -100) {
      data.rotate_CW = 0;
      data.rotate_CCW = 1;
      // } else if (speed_X < -50 && speed_X >= -80) {
      //   data.rotate = -2;
      // } else if (speed_X < -80 && speed_X >= -100) {
      //   data.rotate = -3;
    }
    // Direction Clockwise
    else if (speed_X > 10 && speed_X <= 100) {
      data.rotate_CW = 1;
      data.rotate_CCW = 0;
      // } else if (speed_X > 40 && speed_X <= 70) {
      //   data.rotate = 2;
      // } else if (speed_X > 70 && speed_X <= 100) {
      //   data.rotate = 3;
    }
  }


  //Categorize 3 speed levels for BENDING in both directions (Y-values)
  //But only Drive Y if it is bigger than X
  if (abs(speed_Y) > abs(speed_X)) {
    data.rotate_CW = 0;
    data.rotate_CCW = 0;
    if (speed_Y < -10 && speed_Y >= -100) {
      data.bend_for = 0;
      data.bend_back = 1;
    }
    // Direction Clockwise
    else if (speed_Y > 10 && speed_Y <= 100) {
      data.bend_for = 1;
      data.bend_back = 0;
    }
  }

  if (
    (data.Up_button != prev_Up_button_state) || (data.Left_button != prev_Left_button_state) || (data.Down_button != prev_Down_button_state) || (data.Right_button != prev_Right_button_state) || (data.E_button != prev_E_button_state) || (data.F_button != prev_F_button_state) || (data.Joy_button != prev_Joy_button_state) || (data.Extra_button != prev_Extra_button_state) || (data.rotate_CW != prev_rotate_CW_state) || (data.rotate_CCW != prev_rotate_CCW_state) || (data.bend_for != prev_bend_for_state) || (data.bend_back != prev_bend_back_state)|| (data.E_stop_button != prev_E_stop_button_state)) {
    //only sent data if only one button is pressed simultaneously
    if ((data.Up_button + data.Right_button + data.Down_button + data.Left_button + data.E_button + data.F_button + data.Joy_button + data.Extra_button + data.rotate_CW + data.rotate_CCW + data.bend_for + data.bend_back) <= 1) {
      prev_Up_button_state = data.Up_button;
      prev_Left_button_state = data.Left_button;
      prev_Down_button_state = data.Down_button;
      prev_Right_button_state = data.Right_button;
      prev_E_button_state = data.E_button;
      prev_F_button_state = data.F_button;
      prev_Joy_button_state = data.Joy_button;
      prev_Extra_button_state = data.Extra_button;
      prev_E_stop_button_state = data.E_stop_button;
      prev_rotate_CW_state = data.rotate_CW;
      prev_rotate_CCW_state = data.rotate_CCW;
      prev_bend_for_state = data.bend_for;
      prev_bend_back_state = data.bend_back;


      Serial.print(data.Up_button);
      Serial.print(data.Right_button);
      Serial.print(data.Down_button);
      Serial.print(data.Left_button);
      Serial.print(data.E_button);
      Serial.print(data.F_button);
      Serial.print(data.Joy_button);
      Serial.print(data.Extra_button);
      Serial.print(data.E_stop_button);
      Serial.print(data.rotate_CW);
      Serial.print(data.rotate_CCW);
      Serial.print(data.bend_for);
      Serial.print(data.bend_back);
      Serial.println("\t sent");

      // Send the updated data using radio.write()
      radio.write(&data, sizeof(data));
    }
  } else if (currentTime - lastHandshakeTime >= handshakeInterval) {
    lastHandshakeTime = currentTime;
    if (data.Handshake == 1) {
      data.Handshake = 0;
      radio.write(&data, sizeof(data));
    } else if (data.Handshake == 0) {
      data.Handshake = 1;
      radio.write(&data, sizeof(data));
    }
  }
}